﻿Imports System.Collections.ObjectModel
Imports System.Drawing.Printing
Imports System.IO
Imports ThinkGeo.MapSuite.Layers
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Styles
Imports ThinkGeo.MapSuite.WinForms
Imports ThinkGeo.MapSuite

Public Class Form1
	Private Shared msPdf995IniFileName As String = String.Empty
	Private moRoadsLayer As ShapeFileFeatureLayer
	Private moHydroLayer As ShapeFileFeatureLayer
	Private moPolyLayer As ShapeFileFeatureLayer
	Private moFeatureExtent As RectangleShape
	Private moStyle1 As AreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Blue, GeoColor.SimpleColors.Black, 6)
	Private moStyle2 As AreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Red, GeoColor.SimpleColors.Black, 6)
	Private mbStyle1 As Boolean = True

	Private Shared msPdfFileName As String = String.Empty
	Private Shared msFlagFilePath As String = String.Empty
	Private Shared msPdfSyncFileName As String = String.Empty
	Private Shared msGeoEncode As String = String.Empty
	Private Const mcsPdf995FileName As String = "\Pdf995\Res\Pdf995.ini"
	Private Const mcsPdf995SyncFileName As String = "\Pdf995\PdfSync.ini"
	Private Const mcsProcessPdf As String = "\DeletePdfFlag.exe"
	Private Const mcsFlagFileName As String = "PdfFlag.xyz"
	Private Shared WithEvents moPdfWorker As System.ComponentModel.BackgroundWorker
	Public Const csQuickPdfLicenseKey As String = "jo5m36tw9qc9sw5h49bg7mt4y"


	Public Enum EnumPagePosition
		TopLeft = 1
		TopRight = 2
		LowerLeft = 3
		LowerRight = 4
	End Enum
	Public Enum EnumCenterType
		Horizontal = 1
		Vertical = 2
	End Enum
	Private Sub AddCenteredTitleLabel()
		Dim oLabelPrinterLayer As LabelPrinterLayer = New LabelPrinterLayer()
		Dim oPageBoundingbox As RectangleShape

		'***** Setup the text and the font..
		With oLabelPrinterLayer
			.Text = "Page Layout"
			.Font = New GeoFont("Arial", 10, DrawingFontStyles.Bold)
			.TextBrush = New GeoSolidBrush(GeoColor.StandardColors.Black)

			'***** Set the labels position so that is it centered on the page one inch from the top                        
			oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
			Dim oLabelCenter As PointShape = New PointShape()
			oLabelCenter.X = oPageBoundingbox.UpperRightPoint.X - oPageBoundingbox.Width / 2
			oLabelCenter.Y = oPageBoundingbox.UpperLeftPoint.Y - 0.75

			.PrinterWrapMode = PrinterWrapMode.AutoSizeText
			.ResizeMode = PrinterResizeMode.MaintainAspectRatio
			.SetPosition(5, 0.5, oLabelCenter, PrintingUnit.Inch)
		End With
		Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		oPrinterOverlay.PrinterLayers.Add(Guid.NewGuid.ToString, oLabelPrinterLayer)

	End Sub

	Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

		CreateDefaultLayout()

	End Sub

	Public Sub CreateDefaultLayout()
		Dim oExtentRectangle As RectangleShape
		Dim moPrinterOverlay As PrinterInteractiveOverlay
		Dim moPrinterPage As PagePrinterLayer

		moPrinterOverlay = New PrinterInteractiveOverlay()
		With PageLayout
			'***** Setup the map unit, you want to always use feet or meters for printer layout maps
			.MapUnit = GeographyUnit.Feet
			'***** Create the default zoom levels for the layout map
			.ZoomLevelSet = New PrinterZoomLevelSet(.MapUnit, PrinterHelper.GetPointsPerGeographyUnit(.MapUnit))

			'***** Set the background color to gray so there is contrast with the white page
			.BackgroundOverlay.BackgroundBrush = New GeoSolidBrush(GeoColor.StandardColors.LightGray)

			'***** Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
			'***** The interactive overlay allows the layers to be interacted with
			.InteractiveOverlays.Add("PrinterOverlay", moPrinterOverlay)
			.InteractiveOverlays.MoveToBottom("PrinterOverlay")

			'***** Create the PagePrinterLayer which shows the page boundary and 
			'***** is the area the user will arrange all of the other layers on top of.
			moPrinterPage = New PagePrinterLayer()
			With moPrinterPage
				.Open()
				.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)
				moPrinterOverlay.PrinterLayers.Add("PrinterPage", moPrinterPage)
				'***** Get the page's extent, scale it up by 10%
				oExtentRectangle = CType(RectangleShape.ScaleUp(.GetPosition(), 10), RectangleShape)
				'***** Set oExtentRectangle as the default current extent
			End With
			.CurrentExtent = oExtentRectangle

			'***** Set the minimum scale of the map to the last zoom level
			.MinimumScale = .ZoomLevelSet.ZoomLevel20.Scale

			'***** Add a container for the map layers
			AddMapLayerContainer(mGetPageLayoutMap)

			'***** Add a layer to the page layout
			AddMapLayers(mGetPageLayoutMap)

			'***** Add a scale line
			''CreateDefaultScaleLine(mGetPageLayoutMap)

			'***** Add a scale bar
			CreateDefaultScaleBar(mGetPageLayoutMap)

		End With
	End Sub
	Public Sub AddMapLayers(moPageLayoutMap As MapPrinterLayer)
		Dim sPath As String
		moPageLayoutMap.MapUnit = GeographyUnit.Meter

		sPath = "..\..\Data\WildBirds.shp"
		moPolyLayer = New ShapeFileFeatureLayer(sPath, GeoFileReadWriteMode.Read)
		moPageLayoutMap.Layers.Add(moPolyLayer)
		Dim oColor As New GeoColor(100, GeoColor.GeographicColors.Sand)
		Dim oAreaStyle As AreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Blue, GeoColor.SimpleColors.Black, 1)
		Dim oImage As New GeoImage("..\..\Data\DefaultTexture.tif")
		Dim oBrush As New GeoTextureBrush(oImage)
		oAreaStyle.Advanced.FillCustomBrush = oBrush




		moPolyLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = oAreaStyle
		moPolyLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

		moPolyLayer.Open()
		moPageLayoutMap.MapExtent = moPolyLayer.GetBoundingBox


	End Sub
	Public Sub AddMapLayerContainer(moPageLayoutMap As MapPrinterLayer)
		Dim oPageBoundingbox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)

		moPageLayoutMap = New MapPrinterLayer()

		moPageLayoutMap.MapUnit = GeographyUnit.Feet

		'***** Create the MapPrinterLayer and set the position
		moPageLayoutMap.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)

		'***** Set the maps position slightly below the pages center and 7.5 inches wide and 9 inches tall
		oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
		moPageLayoutMap.SetPosition(7.5, 9, oPageBoundingbox.GetCenterPoint().X, oPageBoundingbox.GetCenterPoint().Y - 0.25, PrintingUnit.Inch)

		'***** Add the MapPrinterLayer to the PrinterInteractiveOverlay
		Dim moPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		moPrinterOverlay.PrinterLayers.Add("MapPrinterLayer", moPageLayoutMap)
	End Sub
	Private Function GetPageBoundingBox(ByVal unit As PrintingUnit) As RectangleShape
		Dim moPrinterPage As PagePrinterLayer = mGetPrinterPage()
		Return moPrinterPage.GetPosition(unit)
	End Function
	Public Sub CreateDefaultScaleLine(moPageLayoutMap As MapPrinterLayer)
		Dim moPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		If moPrinterOverlay.PrinterLayers.Contains("ScaleLine") Then
			moPrinterOverlay.PrinterLayers.Remove("ScaleLine")
		End If
		Dim moScaleLineLayer As ScaleLinePrinterLayer
		'***** Create the scaleLinePrinterLayer and pass in the MapPrinterLayer
		moScaleLineLayer = New ScaleLinePrinterLayer(moPageLayoutMap)

		moScaleLineLayer.MapUnit = moPageLayoutMap.MapUnit
		''moScaleLineLayer.IsVisible = False
		moScaleLineLayer.Open()
		mSetPosition(2.5, 0.25, moScaleLineLayer, EnumPagePosition.LowerLeft, mGetPageLayoutMap)

		'***** Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
		moPrinterOverlay.PrinterLayers.Add("ScaleLine", moScaleLineLayer)
	End Sub
	Public Sub CreateDefaultScaleBar(moPageLayoutMap As MapPrinterLayer)
		Dim moPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		If moPrinterOverlay.PrinterLayers.Contains("ScaleBar") Then
			moPrinterOverlay.PrinterLayers.Remove("ScaleBar")
		End If
	End Sub
	Public Sub mSetPosition(ByVal dWidth As Double, ByVal dHeight As Double, ByVal oPrinterLayer As PrinterLayer, ByVal eInitialPosition As EnumPagePosition, moPageLayoutMap As MapPrinterLayer)
		Dim dCenterPointX As Double
		Dim dCenterPointY As Double
		Dim dDistanceFromEdge As Double = 0.25

		'***** Get the bounding box for the MapPrinterLayer
		moPageLayoutMap.Open()
		Dim oMapBoundingBox As RectangleShape = moPageLayoutMap.GetPosition(PrintingUnit.Inch)
		Select Case eInitialPosition
			Case EnumPagePosition.LowerRight
				dCenterPointX = oMapBoundingBox.LowerRightPoint.X - (dWidth / 2) - dDistanceFromEdge
				dCenterPointY = oMapBoundingBox.LowerRightPoint.Y + (dHeight / 2) + dDistanceFromEdge
			Case EnumPagePosition.LowerLeft
				dCenterPointX = oMapBoundingBox.LowerLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
				dCenterPointY = oMapBoundingBox.LowerLeftPoint.Y + (dHeight / 2) + dDistanceFromEdge
			Case EnumPagePosition.TopRight
				dCenterPointX = oMapBoundingBox.UpperRightPoint.X - (dWidth / 2) - dDistanceFromEdge
				dCenterPointY = oMapBoundingBox.UpperRightPoint.Y - (dHeight / 2) - dDistanceFromEdge
			Case EnumPagePosition.TopLeft
				dCenterPointX = oMapBoundingBox.UpperLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
				dCenterPointY = oMapBoundingBox.UpperLeftPoint.Y - (dHeight / 2) - dDistanceFromEdge
		End Select

		'***** Now let's try to place this layer in the lower right corner of the page
		oPrinterLayer.SetPosition(dWidth, dHeight, dCenterPointX, dCenterPointY, PrintingUnit.Inch)
	End Sub
	Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefresh.Click
		mRefreshMapLayers(mGetPageLayoutMap)
	End Sub
	Private Sub mRefreshMapLayers(moPageLayoutMap As MapPrinterLayer)
		If Not moPageLayoutMap Is Nothing Then
			moPageLayoutMap.IsDrawing = True
			''moPageLayoutMap.ClearCache()
		End If

		PageLayout.Refresh()

		If Not moPageLayoutMap Is Nothing Then
			moPageLayoutMap.IsDrawing = False
		End If
	End Sub

	Private Sub Form1_Shown(sender As Object, e As System.EventArgs) Handles Me.Shown
		Me.PageLayout.Refresh()
	End Sub

	Private Function mGetPrinterOverlay() As PrinterInteractiveOverlay
		For Each oPrinterOverlay In PageLayout.InteractiveOverlays
			If TypeOf oPrinterOverlay Is PrinterInteractiveOverlay Then
				Return CType(oPrinterOverlay, PrinterInteractiveOverlay)
			End If
		Next oPrinterOverlay
		Return Nothing
	End Function
	Private Function mGetPageLayoutMap() As MapPrinterLayer
		Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		Dim oPrinterLayer As PrinterLayer
		For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
			If TypeOf oPrinterLayer Is MapPrinterLayer Then
				Return CType(oPrinterLayer, MapPrinterLayer)
			End If
		Next oPrinterLayer
		Return Nothing
	End Function
	Private Function mGetPrinterPage() As PagePrinterLayer
		Dim oPrinterOverlay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		Dim oPrinterLayer As PrinterLayer
		For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
			If TypeOf oPrinterLayer Is PagePrinterLayer Then
				Return CType(oPrinterLayer, PagePrinterLayer)
			End If
		Next oPrinterLayer
		Return Nothing
	End Function
	Public Sub Print()

		Dim oPrinterSettings As New PrinterSettings()
		oPrinterSettings = GetPrinter()

		Dim oPrinterOverLay As PrinterInteractiveOverlay = mGetPrinterOverlay()
		Dim oPagePrinterLayer As PagePrinterLayer = mGetPrinterPage()

		Dim oPrintDocument As New PrintDocument()
		With oPrintDocument
			.PrinterSettings = oPrinterSettings
			.DefaultPageSettings.Landscape = True
			If oPagePrinterLayer.Orientation = PrinterOrientation.Portrait Then
				.DefaultPageSettings.Landscape = False
			End If
			.DefaultPageSettings.PaperSize = GetPrintPreviewPaperSize(oPagePrinterLayer)
		End With

		Dim oPrinterCanvas As PrinterGeoCanvas = New PrinterGeoCanvas()

		'***** Create a rectangle to represent the drawing area for the document
		oPrinterCanvas.DrawingArea = mGetPrinterCanvasDrawingArea(oPrintDocument)
		'***** Start the drawing process
		oPrinterCanvas.BeginDrawing(oPrintDocument, oPagePrinterLayer.GetBoundingBox(), PageLayout.MapUnit)

		'***** Loop through all of the PrintingLayers in the PrinterInteractiveOverlay 
		'***** and print all of the except for the PagePrinterLayer                
		Dim labelsInAllLayers As Collection(Of SimpleCandidate) = New Collection(Of SimpleCandidate)()
			For Each oPrinterLayer As PrinterLayer In oPrinterOverLay.PrinterLayers
				oPrinterLayer.IsDrawing = True

				If Not (TypeOf oPrinterLayer Is PagePrinterLayer) Then
					oPrinterLayer.Draw(oPrinterCanvas, labelsInAllLayers)
				End If

				oPrinterLayer.IsDrawing = False
			Next oPrinterLayer
		Try
			oPrinterCanvas.EndDrawing()
		Catch ex As Exception
			MessageBox.Show("Error: " + ex.Message, "Boom!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
		End Try

	End Sub

	Private Function mGetPrinterCanvasDrawingArea(oPrintDocument As PrintDocument) As Rectangle
		Dim oRectangle As Rectangle
		'***** Here we create a System.Drawing.Rectangle to be used as the drawing
		'***** area for the PrinterGeoCanvas that will draw the map on the printer.
		'***** Notice that when the page has been set to Landscape the Width and Height
		'***** must be reversed.
		With oPrintDocument.DefaultPageSettings.PrintableArea()
			If oPrintDocument.DefaultPageSettings.Landscape Then
				oRectangle = New Rectangle(0, 0, Convert.ToInt32(.Height), Convert.ToInt32(.Width))
			Else
				oRectangle = New Rectangle(0, 0, Convert.ToInt32(.Width), Convert.ToInt32(.Height))
			End If
		End With

		Return oRectangle
	End Function

	Public Shared Function GetPrinter() As PrinterSettings
		Dim oPrintDialog As New PrintDialog
		Dim oPrinterSettings As PrinterSettings = Nothing

		With oPrintDialog
			If Not .ShowDialog() = DialogResult.Cancel Then
				oPrinterSettings = .PrinterSettings
			End If
		End With
		Return oPrinterSettings
	End Function
	Private Function GetPrintPreviewPaperSize(ByVal pagePrinterLayer As PagePrinterLayer) As PaperSize
		Dim printPreviewPaperSize As PaperSize = New PaperSize("AnsiA", 850, 1100)
		Select Case pagePrinterLayer.PageSize
			Case PrinterPageSize.AnsiA
				printPreviewPaperSize = New PaperSize("AnsiA", 850, 1100)
			Case PrinterPageSize.AnsiB
				printPreviewPaperSize = New PaperSize("AnsiB", 1100, 1700)
			Case PrinterPageSize.AnsiC
				printPreviewPaperSize = New PaperSize("AnsiC", 1700, 2200)
			Case PrinterPageSize.AnsiD
				printPreviewPaperSize = New PaperSize("AnsiD", 2200, 3400)
			Case PrinterPageSize.AnsiE
				printPreviewPaperSize = New PaperSize("AnsiE", 3400, 4400)
			Case PrinterPageSize.Custom
				printPreviewPaperSize = New PaperSize("Custom Size", CInt(pagePrinterLayer.CustomWidth), CInt(pagePrinterLayer.CustomHeight))
			Case Else
		End Select

		Return printPreviewPaperSize
	End Function

	Private Sub btnPrint_Click(sender As System.Object, e As System.EventArgs) Handles btnPrint.Click
		btnPrint.Enabled = False
		Print()
		btnPrint.Enabled = True
	End Sub

End Class
